-- This MySQL script converts MOVES emission rates into a Seashr EMISFACT table
-- suitable for an AERMOD input file.
-- 
-- Users should consult Section 4 of EPA's "Transportation Conformity Guidance for 
-- Quantitative Hot-spot Analyses in PM2.5 and PM10 Nonattainment and Maintenance Areas"
-- for guidance on filling out the MOVES RunSpec and importing the appropriate inputs.  
-- 
-- Runs should be completed for each of 16 representative time periods and results 
-- located in a single output database.  These 16 runs should correspond with the 
-- recommended time periods: January, April, July, and October; 
-- and hours 1, 7, 13, and 19.
--       
-- ***********************************************************
-- The user *must* have a mapping of links to sources in a .csv file called
-- `Links_Sources.csv` located in folder `c:\MOVES2AERMOD input data`.
-- The user *must* have link area information in a .csv file called
-- `Source_Area.csv` located in folder `c:\MOVES2AERMOD input data`.
-- The user *must* have a diurnal/seasonal traffic distribution in 
--  a .csv file called `Traffic_Distribution.csv` located in folder
--  `c:\MOVES2AERMOD input data`.
-- ***********************************************************
--                          
-- The script may take several minutes to run.
--    
-- The EMISFACT table produced in the output database 
-- is called AERMOD_output.
--                
--                
-- Script version: June 2012.




Drop   table if exists  uLinkSources;
create table  uLinkSources
  ( AERMODsourceId char(3) default NULL,
    MOVESlinkId    int     default NULL
  ) ENGINE=MyISAM;

load data infile 'C:/MOVES2Aermod Input Data/Links_Sources.csv'
into table uLinkSources
fields terminated by ','
optionally enclosed by '"'
escaped by '\\'
lines terminated by '\r\n'
ignore 1 lines;




Drop   table if exists uTrafDist;
Create table uTrafDist
  (  seasonId     int,
     hourId       int,
     distribution int
  ) ENGINE=MyISAM;

load data infile 'C:/MOVES2Aermod Input Data/Traffic_Distribution.csv'
into table uTrafDist
fields terminated by ','
optionally enclosed by '"'
escaped by '\\'
lines terminated by '\r\n'
ignore 1 lines;


Alter table uTrafDist add column rowId int;
Update      uTrafDist set rowId =  1 Where seasonId >=  1 and hourId >=  1;
Update      uTrafDist set rowId =  2 Where seasonId >=  1 and hourId >=  9;
Update      uTrafDist set rowId =  3 Where seasonId >=  1 and hourId >= 17;
Update      uTrafDist set rowId =  4 Where seasonId >=  4 and hourId >=  1;
Update      uTrafDist set rowId =  5 Where seasonId >=  4 and hourId >=  9;
Update      uTrafDist set rowId =  6 Where seasonId >=  4 and hourId >= 17;
Update      uTrafDist set rowId =  7 Where seasonId >=  7 and hourId >=  1;
Update      uTrafDist set rowId =  8 Where seasonId >=  7 and hourId >=  9;
Update      uTrafDist set rowId =  9 Where seasonId >=  7 and hourId >= 17;
Update      uTrafDist set rowId = 10 Where seasonId >= 10 and hourId >=  1;
Update      uTrafDist set rowId = 11 Where seasonId >= 10 and hourId >=  9;
Update      uTrafDist set rowId = 12 Where seasonId >= 10 and hourId >= 17;

Alter table uTrafDist add column colId int;
Update      uTrafDist set colId = 1 where hourId in ( 1, 9,17);
Update      uTrafDist set colId = 2 where hourId in ( 2,10,18);
Update      uTrafDist set colId = 3 where hourId in ( 3,11,19);
Update      uTrafDist set colId = 4 where hourId in ( 4,12,20);
Update      uTrafDist set colId = 5 where hourId in ( 5,13,21);
Update      uTrafDist set colId = 6 where hourId in ( 6,14,22);
Update      uTrafDist set colId = 7 where hourId in ( 7,15,23);
Update      uTrafDist set colId = 8 where hourId in ( 8,16,24);


Alter table uTrafDist add column xMonth       int;
Alter table uTrafDist add column xHour        int;

Update      uTrafDist set        xMonth = Distribution/100;
Update      uTrafDist set        xHour  = Mod(Distribution,100);


Drop   table if exists LinkTraf;
Create table LinkTraf
Select a.*,
       b.*
From   uLinkSources as a
Join   uTrafDist    as b;



-- table 3, SourceArea:

Drop   table if exists uSourceArea;
Create table uSourceArea
  (  linkId int,
     area   real
  ) ENGINE=MyISAM;

load data infile 'C:/MOVES2Aermod Input Data/Source_Area.csv'
into table uSourceArea
fields terminated by ','
optionally enclosed by '"'
escaped by '\\'
lines terminated by '\r\n'
ignore 1 lines;


Drop   table if exists pm25_grams_per_hour;
Create table           pm25_grams_per_hour
Select   movesRunId,
         yearId,
         monthId,
         hourId,
         linkId,
         'Total PM2.5' as pollutantId,
         sum(emissionQuant) as GramsPerHour
From     movesOutput
Where    pollutantId in (110, 116, 117)
  and    linkId -- < 3
Group by movesRunId,
         yearId,
         monthId,
         hourId,
         linkId;


Drop   table if exists AERMODsourceID_rates;
Create table           AERMODsourceID_rates
Select a.*,
       b.AERMODSourceId  -- table 1
From   pm25_grams_per_hour as a
Join   uLinkSources        as b on a.linkId = b.MOVESlinkId;




Alter table AERMODsourceID_rates add column gramsSec   real;
Alter table AERMODsourceID_rates add column sourceArea int;
Alter table AERMODsourceID_rates add column gramsSecM2 real;

Update      AERMODsourceID_rates as a set a.sourceArea = (Select b.area
                                                          From   uSourceArea as b
                                                          Where  a.linkId = b.linkId);

Update      AERMODsourceID_rates      set   gramsSec   = GramsPerHour / 3600;

Update      AERMODsourceID_rates      set   gramsSecM2 = gramsSec / sourceArea;



Alter table   linkTraf add column xGramsSecM2 real;

-- Update        linkTraf as a set xGramsSecM2 = (Select count(*)
--                                                From   AERMODsourceID_rates as b
--                                                Where  a.xMonth         = b.monthId
--                                                  and  a.xHour          = b.hourId
--                                                  and  a.AERMODsourceId = b.AERMODsourceId );

Update        linkTraf as a set xGramsSecM2 = (Select gramsSecM2
                                               From   AERMODsourceID_rates as b
                                               Where  a.xMonth         = b.monthId
                                                 and  a.xHour          = b.hourId
                                                 and  a.AERMODsourceId = b.AERMODsourceId );








Drop   table if exists just12;
Create table just12
  (rowId int);

Insert into just12 (rowId) values
 (1), (2), (3), ( 4), ( 5), ( 6),
 (7), (8), (9), (10), (11), (12);



Drop   table if exists EMISFACT_traffic1;
Create table EMISFACT_traffic1
Select a.AERMODsourceId,
       a.MOVESLinkId,
       b.rowId
From   uLinkSources as a
Join   just12       as b;



Drop   table if exists EMISFACT_traffic2;
Create table EMISFACT_traffic2
Select   a.AERMODsourceId,
         a.MOVESLinkId,
         a.rowId,

         (Select b.distribution From uTrafDist as b Where b.rowId = a.rowId and b.colId = 1) as c1,
         (Select c.distribution From uTrafDist as c Where c.rowId = a.rowId and c.colId = 2) as c2,
         (Select d.distribution From uTrafDist as d Where d.rowId = a.rowId and d.colId = 3) as c3,
         (Select e.distribution From uTrafDist as e Where e.rowId = a.rowId and e.colId = 4) as c4,
         (Select f.distribution From uTrafDist as f Where f.rowId = a.rowId and f.colId = 5) as c5,
         (Select g.distribution From uTrafDist as g Where g.rowId = a.rowId and g.colId = 6) as c6,
         (Select h.distribution From uTrafDist as h Where h.rowId = a.rowId and h.colId = 7) as c7,
         (Select i.distribution From uTrafDist as i Where i.rowId = a.rowId and i.colId = 8) as c8

From     EMISFACT_traffic1 as a
Order by a.AERMODsourceId,
         a.rowId;


Drop   table if exists EMISFACT_mapping1;
Create table EMISFACT_mapping1
Select a.AERMODsourceId,
       a.MOVESLinkId,
       b.rowId
From   uLinkSources as a
Join   just12       as b;



Drop   table if exists EMISFACT_mapping;
Create table EMISFACT_mapping
Select   a.AERMODsourceId,
         a.MOVESLinkId,
         a.rowId,

         (Select xGramsSecM2 from LinkTraf as b where b.AERMODsourceId = a.AERMODsourceId
                                                  and b.rowId          = a.rowId
                                                  and b.colId          = 1                )  as c1,

         (Select xGramsSecM2 from LinkTraf as c where c.AERMODsourceId = a.AERMODsourceId
                                                  and c.rowId          = a.rowId
                                                  and c.colId          = 2                )  as c2,

         (Select xGramsSecM2 from LinkTraf as d where d.AERMODsourceId = a.AERMODsourceId
                                                  and d.rowId          = a.rowId
                                                  and d.colId          = 3                )  as c3,

         (Select xGramsSecM2 from LinkTraf as e where e.AERMODsourceId = a.AERMODsourceId
                                                  and e.rowId          = a.rowId
                                                  and e.colId          = 4                )  as c4,

         (Select xGramsSecM2 from LinkTraf as f where f.AERMODsourceId = a.AERMODsourceId
                                                  and f.rowId          = a.rowId
                                                  and f.colId          = 5                )  as c5,

         (Select xGramsSecM2 from LinkTraf as g where g.AERMODsourceId = a.AERMODsourceId
                                                  and g.rowId          = a.rowId
                                                  and g.colId          = 6                )  as c6,

         (Select xGramsSecM2 from LinkTraf as h where h.AERMODsourceId = a.AERMODsourceId
                                                  and h.rowId          = a.rowId
                                                  and h.colId          = 7                )  as c7,

         (Select xGramsSecM2 from LinkTraf as i where i.AERMODsourceId = a.AERMODsourceId
                                                  and i.rowId          = a.rowId
                                                  and i.colId          = 8                )  as c8

From     EMISFACT_mapping1 as a
Order by a.AERMODsourceId,
         a.rowId;


-- Select * from EMISFACT_mapping;


Drop   table if exists AERMOD_Output;
Create table AERMOD_Output
Select  -- MOVESLINKId,
           'SO',
           'EMISFACT',
           AERMODsourceId,
           'SEASHR',
        -- RowId,
           c1, c2, c3, c4, c5, c6, c7, c8
From       EMISFACT_mapping
Order by   MOVESLINKId,
           AERMODsourceId,
           RowId;



-- Select * from AERMOD_Output;


Drop table if exists uLinkSources;
Drop table if exists uTrafDist;
Drop table if exists LinkTraf;
Drop table if exists uSourceArea;
Drop table if exists AERMODsourceID_rates;
Drop table if exists just12;
Drop table if exists EMISFACT_traffic1;
Drop table if exists EMISFACT_traffic2;

